"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var child_process_1 = require("child_process");
module.exports = function (
// Object of sites
sites, 
// Specific devices if different to default
inputDevices) {
    if (inputDevices === void 0) { inputDevices = utils_1.defaultDevices; }
    // Collate PLaywright projects
    var projects = [];
    // Loop through defined sites
    for (var _i = 0, sites_1 = sites; _i < sites_1.length; _i++) {
        var site = sites_1[_i];
        // Reset the devices
        var devices = inputDevices;
        // Override the devices if necessary
        if (site.devices) {
            devices = site.devices;
            delete site.devices;
        }
        // Do we have a number of devices?
        if (devices.length > 0) {
            // Send the first device & search for unit, spec & test files
            projects.push((0, utils_1.convertSiteToPlaywrightProject)(site, devices.shift()));
            // Create a site for each remaining device & only search for `.test` files
            for (var _a = 0, _b = (devices !== null && devices !== void 0 ? devices : []); _a < _b.length; _a++) {
                var device = _b[_a];
                projects.push((0, utils_1.convertSiteToPlaywrightProject)(site, device, { testsToFind: 'test' }));
            }
        }
        else {
            projects.push((0, utils_1.convertSiteToPlaywrightProject)(site));
        }
    }
    // Return Playwright configuration
    return {
        /* Run tests in files in parallel */
        fullyParallel: true,
        /* Fail the build on CI if you accidentally left test.only in the source code. */
        forbidOnly: !!process.env.CI,
        quiet: !!process.env.CI,
        /* Retry on CI only */
        retries: process.env.CI ? 2 : 0,
        /* Opt out of parallel tests on CI. */
        workers: process.env.CI ? 1 : undefined,
        /* Shared settings for all the projects below. See https://playwright.dev/docs/api/class-testoptions. */
        use: {
            /* Collect trace when retrying the failed test. See https://playwright.dev/docs/trace-viewer */
            trace: 'on-first-retry',
            screenshot: 'only-on-failure'
        },
        /* Configure projects for major browsers */
        projects: projects,
        /* Reporter to use. See https://playwright.dev/docs/test-reporters */
        reporter: (process.env.CI ?
            [
                ['line'],
                [
                    'junit',
                    {
                        outputFile: './test-results/playwright-report.xml'
                    }
                ]
            ] :
            [
                [
                    'list',
                    {
                        printSteps: true
                    }
                ],
                [
                    'monocart-reporter',
                    {
                        name: 'Playwright Report',
                        outputFile: './test-results/report.html',
                        customFieldsInComments: true,
                        mermaid: {
                            scriptSrc: 'https://cdn.jsdelivr.net/npm/mermaid@latest/dist/mermaid.min.js',
                            // mermaid config: https://mermaid.js.org/config/schema-docs/config.html
                            config: {
                                startOnLoad: false
                            }
                        },
                        tags: {
                            vr: {
                                style: {
                                    background: '#6e178f'
                                },
                                description: '"Visual Regression (VR)" is a technology that allows you to test the visual appearance of a web page'
                            },
                            snapshot: {
                                style: {
                                    background: '#ff8f00'
                                },
                                description: 'Snapshots are tests which require saving of an initial state of a web page'
                            },
                        },
                        columns: function (defaultColumns) {
                            // add `device` column after `duration` column
                            var index = defaultColumns.findIndex(function (column) { return column.id === 'duration'; });
                            defaultColumns.splice(index, 0, {
                                id: 'label',
                                name: 'Label',
                                width: 150,
                                formatter: function (v) { return v !== null && v !== void 0 ? v : ''; }
                            }, {
                                id: 'env',
                                name: 'Environment',
                                width: 150,
                                formatter: function (v) { var _a; return (_a = (v.charAt(0).toUpperCase() + v.slice(1))) !== null && _a !== void 0 ? _a : ''; }
                            }, {
                                id: 'device',
                                name: 'Device',
                                width: 150,
                                formatter: function (v) { return v !== null && v !== void 0 ? v : ''; }
                            });
                            defaultColumns.push({
                                id: 'description',
                                name: 'Description',
                                markdown: true,
                                searchable: true
                            });
                        },
                        visitor: function (data, pwData) {
                            // add `device` for project data
                            if (data.type === 'suite' && data.suiteType === 'project') {
                                data.label = data.metadata && data.metadata.label;
                                data.env = data.metadata && data.metadata.env;
                                data.device = data.metadata && data.metadata.device;
                                return;
                            }
                            // add `device` for case data
                            if (data.type === 'case') {
                                var parent_1 = pwData.parent;
                                while (parent_1) {
                                    if (parent_1.project) {
                                        var project = parent_1.project();
                                        if (project.metadata) {
                                            data.label = project.metadata.label;
                                            data.env = project.metadata.env;
                                            data.device = project.metadata.device;
                                        }
                                        break;
                                    }
                                    parent_1 = parent_1.parent;
                                }
                            }
                        },
                        onEnd: function (reportData, helper) { return __awaiter(void 0, void 0, void 0, function () {
                            var hasFailed;
                            return __generator(this, function (_a) {
                                hasFailed = helper.find(function (item) { return item.type === 'case' && item.caseType === 'failed'; });
                                if (hasFailed) {
                                    console.log('There are some of the tests failed, open report ...');
                                    (0, child_process_1.execSync)("npx monocart show-report ".concat(reportData.htmlPath));
                                }
                                return [2 /*return*/];
                            });
                        }); }
                    }
                ]
            ])
    };
};
